_AddPermissions( "Bluetooth,Location" );

//Add CreateMicroBit method to global app object.
app.CreateBluetoothLE = function( options ) 
{ 
    return new BluetoothLE( options );
}

//Plugin wrapper class.
function BluetoothLE( options )
{
    var _this = this;
    var ctrl=null, onConnect=null, onSelect=null, beacon=false;
    var dlgScan, dlgLst, tScan, tProg, prog=0, name="", address="", devices="", filter=null;
    
    this.plg = _CreatePlugin( "org.droidscript.plugins.BluetoothLE", options );  
	this.GetVersion = function( num, txt ) { return parseFloat( this.plg.Send( "GetVersion" ) ); }
    this.StartScan = function() { this.plg.Send( "StartScan" ); }
    this.StopScan = function() { this.plg.Send( "StopScan" ); }
    this.Connect = function( address, options ) { this.plg.Send( "Connect", address, options ); }
    this.Disconnect = function() { this.plg.Send( "Disconnect" ); }
	this.IsConnected = function() { return this.plg.Send("IsConnected")=="true"; }   
    this.SendUart = function( msg, encoding ) { this.plg.Send( "SendUart", msg, encoding ); }
    this.SetUartMode = function( mode ) { this.plg.Send( "SetUartMode", mode ); }
	this.SetUartIds = function( svc,tx,rx ) { this.plg.Send( "SetUartIds", svc,tx,rx ); }
    this.SetSplitMode = function( mode,p1,p2 ) { this.plg.Send( "SetSplitMode", mode,p1,p2 ); }
    this.SetOnUartReceive = function( callback ) { this.plg.Send( "SetOnUartReceive", _Cbm(callback) ); }
    this.SetOnDevice = function( callback ) { this.plg.Send( "SetOnDevice", _Cbm(callback) ); }
    this.SetOnConnect = function( callback ) { this.plg.Send( "SetOnConnect", _Cbm(callback) ); onConnect = callback; }
    this.SetOnDisconnect = function( callback ) { this.plg.Send( "SetOnDisconnect", _Cbm(callback) ); }
    this.SetOnSelect = function( callback ) { onSelect = callback; }
    
	this.SetMtu = function( size ) { return this.plg.Send( "SetMtu", size )=="true"; }
    this.WatchValue = function( svcId,charId,charType,callback ) { this.plg.Send( "WatchValue", svcId,charId,charType,_Cbm(callback) ); }
	this.UnWatchValue = function( svcId,charId ) { this.plg.Send( "UnWatchValue", svcId,charId ); }
	this.ReadValue = function( svcId,charId,charType,callback ) { this.plg.Send( "ReadValue", svcId,charId,charType,_Cbm(callback) ); }
	this.WriteValue = function( svcId,charId,charType,val ) { this.plg.Send( "WriteValue", svcId,charId,charType,val ); }
    
    this.Select = function( address, isBeacon )
    {
        app.SetDebugEnabled( false );
        filter = address;
        beacon = isBeacon
        
        //Create dialog window and layout.
        dlgScan = app.CreateDialog( "Searching..." );
        dlgScan.SetBackColor( "#bb000000" );
        dlgScan.SetOnCancel( _this.OnCancel );
        var lay = app.CreateLayout( "linear", "vertical,fillxy" );
        lay.SetPadding( 0.04, 0.02, 0.04, 0 );
    
        //Create BT list.
        dlgLst = app.CreateList( "", 0.8, 0.4 );
        dlgLst.SetOnTouch( _this.OnChoose );
        lay.AddChild( dlgLst );
        
        //Add dialog layout and show dialog.
        dlgScan.AddLayout( lay );
        dlgScan.Show();
        
        //Start scanning.
        devices = "";
        this.Disconnect();
        this.SetOnDevice( _this.OnDevice );
        this.plg.Send( "SetOnConnect", _Cbm(_this.OnConnect) );
        this.StartScan();
        tScan = setTimeout( StopScan, 30000 );
        tProg = setInterval( ShowProgress, 500 );
        
        app.SetDebugEnabled( true );
    }
    
    function ShowProgress()
    {
        app.SetDebugEnabled( false );
        var dots = "";
        for( var i=0; i<prog; i++ ) dots += ".";
        if( prog++==5 ) prog = 0;
        dlgScan.SetTitle( "Searching"+dots );
        app.SetDebugEnabled( true );
    }
    
    function StopScan()
    {
       _this.StopScan(); 
       clearInterval( tProg );
    }
    
    this.OnDevice = function( name, address, bonded, rssi, data )
    {
        app.SetDebugEnabled( false );
        console.log( "name=" + (beacon?data.url:name) + " address=" + address 
            + " bonded=" + bonded + " rssi=" + rssi + " data=" + JSON.stringify(data) );
        
        if( beacon && !data.url ) { app.SetDebugEnabled(true); return; }
        
        if( filter==null || address.indexOf(filter)>-1 ) 
        {
            if( devices.indexOf(address)==-1 ) {
                devices += (devices?",":"") + address;
                dlgLst.AddItem( beacon?data.url:(name?name:data.type), address );
            }
        }
        app.SetDebugEnabled( true );
    }
    
    this.OnChoose = function( title, body )
    {
        app.SetDebugEnabled( false );
        
        address = body;
        name = title;
      
        clearTimeout( tScan ); 
        clearInterval( tProg );
        _this.StopScan();
        //_this.Connect( address );
        if( onSelect ) onSelect( name, address );
        dlgScan.Dismiss();
        //app.ShowProgress();
        //setTimeout( "app.HideProgress()", 5000 );
        
        app.SetDebugEnabled( true );
    }
    
    this.OnCancel = function()
    {
        clearTimeout( tScan ); 
        clearInterval( tProg );
        _this.StopScan();
        dlgScan.Dismiss();
    }
    
    this.OnConnect = function()
    {
        app.HideProgress();
        console.log( "Connected to " + address );
        if( onConnect ) onConnect();
    }
    
}